/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "ExpCamera.h"
#include "ExpPipelineTypes.h"
#include "MixCastLightComponent.h"
#include "Transmission/Unreal-Interop/SharedQueue/SharedQueueProducer.hpp"
#include "Transmission/Unreal-Interop/SharedTextures/OwnedUnrealSharedTexture_D3D11.h"
#include "Data/SharedTexIds.h"
#include "Data/SdkData.h"
#include "MixCastLog.h"
#include "Utility/StringUtils.hpp"
#include "Utility/TimeUtils.hpp"

#include "CoreMinimal.h"
#include "Engine/World.h"
#include "GameFramework/WorldSettings.h"
#include "Kismet/GameplayStatics.h"
#include "Components/LightComponent.h"
#include "Components/PointLightComponent.h"
#include "Components/DirectionalLightComponent.h"
#include "Components/SpotLightComponent.h"

class MIXCAST_API FExpFrameSender
{
public:
	const int QueueCapacity = 10;

private:
	static TMap<FString, SharedQueueProducer<ExpFrame>*> cachedFrameQueues;

	AExpCamera* cam = NULL;
	FDelegateHandle frameDelegate;

	SharedQueueProducer<ExpFrame>* renderedExpFrames = NULL;
	OwnedUnrealSharedTexture** sharedExpLayersTexs;
	OwnedUnrealSharedTexture** sharedExpOpaqueLayerTexs;

	bool firstWriteCompleted = false;
	bool consumerMissedLastFrame = false;

	void PopulateLights(ExpLight lights[]);

	void CreateQueue();
	void CreateTextureList();
	void DestroyTextureList();

public:
	static void DisposeAllCachedQueues();

	FExpFrameSender(AExpCamera* cam);
	~FExpFrameSender();

	void HandleFrameUpdated(AExpCamera* curCam);
};
