/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "MixCastSliceCanvas.h"

#include "MixCastDefines.h"
#include "Capture/ExpCamera.h"
#include "Data/SdkData.h"

#include "Engine/Canvas.h"
#include "UObject/ConstructorHelpers.h"

UMixCastSliceCanvas::UMixCastSliceCanvas(const FObjectInitializer& ObjectInitializer)
	: Super(ObjectInitializer)
{
#if UE_4_22_OR_LATER
	bNeedsTwoCopies = true;
#endif

	static ConstructorHelpers::FObjectFinder<UMaterial> Find_FullRenderBlitMat(TEXT("Material'/MixCast/TransferOpaque.TransferOpaque'"));
	static ConstructorHelpers::FObjectFinder<UMaterial> Find_TransferWithAlphaChannelMat(TEXT("Material'/MixCast/TransferWithAlphaChannel.TransferWithAlphaChannel'"));
	static ConstructorHelpers::FObjectFinder<UMaterial> Find_TransferWithSeparateAlphaChannelMat(TEXT("Material'/MixCast/TransferWithSeparateAlphaChannel.TransferWithSeparateAlphaChannel'"));
	static ConstructorHelpers::FObjectFinder<UMaterial> Find_TransferAsAdditiveMat(TEXT("Material'/MixCast/TransferAsAdditive.TransferAsAdditive'"));
	static ConstructorHelpers::FObjectFinder<UMaterial> Find_TransferKeyedMat(TEXT("Material'/MixCast/TransferKeyed.TransferKeyed'"));

	if (Find_FullRenderBlitMat.Succeeded())
	{
		//EMaterialAlphaType::Type::Opaque
		fullRenderMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_FullRenderBlitMat.Object),
			this,
			TEXT("Transfer_FullRender_Opaque")
			));
		foregroundMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_FullRenderBlitMat.Object),
			this,
			TEXT("Transfer_Foreground_Opaque")
			));
	}
	else
	{
		fullRenderMats.Add(nullptr);
		foregroundMats.Add(nullptr);
	}
	if (Find_TransferKeyedMat.Succeeded())
	{
		//EMaterialAlphaType::Type::Keyed
		fullRenderMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferKeyedMat.Object),
			this,
			TEXT("Transfer_FullRender_Keyed")
			));
		foregroundMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferKeyedMat.Object),
			this,
			TEXT("Transfer_Foreground_Keyed")
			));
	}
	else
	{
		fullRenderMats.Add(nullptr);
		foregroundMats.Add(nullptr);
	}
	if (Find_TransferAsAdditiveMat.Succeeded())
	{
		//EMaterialAlphaType::Type::ForceAdditive
		fullRenderMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferAsAdditiveMat.Object),
			this,
			TEXT("Transfer_FullRender_ForceAdditive")
			));
		foregroundMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferAsAdditiveMat.Object),
			this,
			TEXT("Transfer_Foreground_ForceAdditive")
			));
	}
	else
	{
		fullRenderMats.Add(nullptr);
		foregroundMats.Add(nullptr);
	}
	if (Find_TransferWithAlphaChannelMat.Succeeded())
	{
		//EMaterialAlphaType::Type::AlphaChannel
		fullRenderMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferWithAlphaChannelMat.Object),
			this,
			TEXT("Transfer_FullRender_AlphaChannel")
			));
		foregroundMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferWithAlphaChannelMat.Object),
			this,
			TEXT("Transfer_Foreground_AlphaChannel")
			));
	}
	else
	{
		fullRenderMats.Add(nullptr);
		foregroundMats.Add(nullptr);
	}
	if (Find_TransferWithSeparateAlphaChannelMat.Succeeded())
	{
		//EMaterialAlphaType::Type::SeparateAlphaChannel
		fullRenderMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferWithSeparateAlphaChannelMat.Object),
			this,
			TEXT("Transfer_FullRender_SeparateAlphaChannel")
		));
		foregroundMats.Add(UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_TransferWithSeparateAlphaChannelMat.Object),
			this,
			TEXT("Transfer_Foreground_SeparateAlphaChannel")
		));
	}
	else
	{
		fullRenderMats.Add(nullptr);
		foregroundMats.Add(nullptr);
	}

	OnCanvasRenderTargetUpdate.AddUniqueDynamic(this, &UMixCastSliceCanvas::BlitLayers);
}

void UMixCastSliceCanvas::Initialize(int layerWidth, int layerHeight, bool separateAlpha)
{
	LayerWidth = layerWidth;
	LayerHeight = layerHeight;
	SeparateAlpha = separateAlpha;

	ClearColor = FColor::Black;	//alpha channel inverted in Unreal render textures so clear with (0,0,0,1)
	bAutoGenerateMips = false;
	TargetGamma = 2.2f;
	InitCustomFormat(layerWidth, layerHeight * 2, PF_FloatRGBA, true);
	UpdateResourceImmediate();

	fullRenderTex = CreateRenderTarget(TEXT("FullRenderRGB"), layerWidth, layerHeight);
	for( int i = 0; i < fullRenderMats.Num(); i++ )
		fullRenderMats[i]->SetTextureParameterValue(TEXT("MainTex"), fullRenderTex);

	foregroundTex = CreateRenderTarget(TEXT("ForegroundRenderRGB"), layerWidth, layerHeight);
	for (int i = 0; i < foregroundMats.Num(); i++)
		foregroundMats[i]->SetTextureParameterValue(TEXT("MainTex"), foregroundTex);

	EMixCastTranslucencyMode::Type translucencyMode = SdkData::GetProjectSettings()->TranslucencyMode.GetValue();
	if (translucencyMode == EMixCastTranslucencyMode::FromAlpha)
	{
		if (separateAlpha)
		{
			if (SdkData::GetProjectSettings()->CanRenderTransparentBg)
			{
				fullAlphaTex = CreateRenderTarget(TEXT("FullRenderA"), layerWidth, layerHeight);
				fullRenderMats[EMaterialAlphaType::Type::SeparateAlphaChannel]->SetTextureParameterValue(TEXT("AlphaTex"), fullAlphaTex);
			}
			foregroundAlphaTex = CreateRenderTarget(TEXT("ForegroundRenderA"), layerWidth, layerHeight);
			foregroundMats[EMaterialAlphaType::Type::SeparateAlphaChannel]->SetTextureParameterValue(TEXT("AlphaTex"), foregroundAlphaTex);
		}
	}
	else if (translucencyMode == EMixCastTranslucencyMode::None)
	{
		FLinearColor keyColor = SdkData::GetProjectSettings()->BackgroundColor;
		keyColor.A = 0;
		fullRenderMats[EMaterialAlphaType::Type::Keyed]->SetVectorParameterValue(FName("KeyColor"), keyColor);
		foregroundMats[EMaterialAlphaType::Type::Keyed]->SetVectorParameterValue(FName("KeyColor"), keyColor);
	}
}

void UMixCastSliceCanvas::BlitLayers(UCanvas* Canvas, int32 Width, int32 Height)
{
	if (blitFull) {
		Canvas->K2_DrawMaterial(
			GetTransferMaterial(fullRenderMats, renderTranslucent, SeparateAlpha),
			FVector2D::ZeroVector,
			FVector2D(LayerWidth, LayerHeight),
			FVector2D::ZeroVector
			);
	}
	if (blitForeground) {
		Canvas->K2_DrawMaterial(
			GetTransferMaterial(foregroundMats, true, SeparateAlpha),
			FVector2D(0, LayerHeight),
			FVector2D(LayerWidth, LayerHeight),
			FVector2D::ZeroVector
			);
	}
}

UTextureRenderTarget2D* UMixCastSliceCanvas::CreateRenderTarget(const TCHAR* name, int width, int height)
{
	UTextureRenderTarget2D* tex = NewObject<UTextureRenderTarget2D>(this, name);
	tex->bAutoGenerateMips = false;
	tex->TargetGamma = 2.2f;
	tex->InitCustomFormat(width, height, PF_FloatRGBA, false);
	tex->UpdateResourceImmediate();
	return tex;
}

UMaterialInstanceDynamic* UMixCastSliceCanvas::GetTransferMaterial(TArray<UMaterialInstanceDynamic*>& mats, 
	bool isTransparent, bool separatedAlpha)
{
	if (!isTransparent)
		return mats[EMaterialAlphaType::Type::Opaque];

	EMixCastTranslucencyMode::Type translucencyMode = SdkData::GetProjectSettings()->TranslucencyMode.GetValue();
	if (translucencyMode == EMixCastTranslucencyMode::FromAlpha) 
	{
		if (separatedAlpha)
			return mats[EMaterialAlphaType::Type::SeparateAlphaChannel];
		else
			return mats[EMaterialAlphaType::Type::AlphaChannel];
	}
	else if (translucencyMode == EMixCastTranslucencyMode::ForceAdditive)
		return mats[EMaterialAlphaType::Type::ForceAdditive];
	else
		return mats[EMaterialAlphaType::Type::Keyed];
}