/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "ExpVideoInputSpawner.h"

#include "MixCast.h"
#include "Data/SdkData.h"
#include "MixCastLog.h"

#include "Async/Async.h"

#include "Camera/CameraComponent.h"
#include "Engine/Engine.h"
#include "Engine/World.h"
#include "GameFramework/PlayerController.h"
#include "HeadMountedDisplay.h"
#include "Kismet/GameplayStatics.h"
#include "HeadMountedDisplay.h"

UExpVideoInputSpawner::UExpVideoInputSpawner(const FObjectInitializer &ObjectInitializer)
	: Super(ObjectInitializer)
{
}

void UExpVideoInputSpawner::SynchronizeSpawnedVideoInputs()
{
	 for (int i = videoInputs.Num() - 1; i >= 0; i--)
	 {
		 bool foundAgain = false;
		 for (int j = 0; j < SdkData::VideoInputs.size() && !foundAgain; j++)
			 foundAgain |= SdkData::VideoInputs[j].identifier.compare(videoInputs[i]->Identifier) == 0;

		 if (!foundAgain)
		 {
			 if (videoInputs[i]->IsValidLowLevel())
				 videoInputs[i]->Destroy();
			 videoInputs.RemoveAt(i);
		 }
	 }

	 for (int i = 0; i < SdkData::VideoInputs.size(); i++)
	 {
		 bool alreadyFound = false;
		 for (int j = 0; j < videoInputs.Num() && !alreadyFound; j++)
			 alreadyFound |= videoInputs[j]->Identifier.compare(SdkData::VideoInputs[i].identifier) == 0;

		 if (!alreadyFound)
		 {
			 UClass* videoInputClass = SdkData::GetProjectSettings()->VideoInputClass.TryLoadClass<AExpVideoInput>();
			 if (videoInputClass == nullptr)
				 videoInputClass = AExpVideoInput::StaticClass();
			 AActor* videoInputActor = UGameplayStatics::BeginDeferredActorSpawnFromClass(this, videoInputClass, FTransform::Identity);
			 AExpVideoInput* videoInput = Cast<AExpVideoInput>(videoInputActor);
			 videoInput->Identifier = SdkData::VideoInputs[i].identifier;
			 videoInput->GetRootComponent()->AttachToComponent(this, FAttachmentTransformRules::KeepRelativeTransform);
			 UGameplayStatics::FinishSpawningActor(videoInputActor, FTransform::Identity);
			 videoInputs.Add(videoInput);
		 }
	 }
}

void UExpVideoInputSpawner::ClearSpawnedVideoInputs()
{
	 for (int i = 0; i < videoInputs.Num(); i++)
	 {
		 if (videoInputs[i]->IsValidLowLevel())
			 videoInputs[i]->Destroy();
	 }
	 videoInputs.Empty();
}